﻿Public Class frmEditField

#Region " Declarations "
  Private _field As Field
#End Region

#Region " Constructors/Teardown "
  Public Sub New(ByVal field As Field, ByVal allowDataEdit As Boolean)
    Const MAX_BYTES_TO_EDIT As Integer = 1024
    InitializeComponent()

    _field = field

    If _field.Id.HasValue Then
      txtFieldId.Text = _field.Id.Value.ToString("X4")

      If _field.Data IsNot Nothing Then
        For i As Integer = 0 To Math.Min(_field.Data.Length, MAX_BYTES_TO_EDIT) - 1
          txtData.Text &= _field.Data(i).ToString("X2")
        Next

        If _field.Data.Length > MAX_BYTES_TO_EDIT Then
          allowDataEdit = False
        End If
      End If
    End If

    If Not allowDataEdit Then
      txtData.Enabled = False
    Else
      txtData.Enabled = True
    End If
  End Sub
#End Region

#Region " Public Properties "
  Public ReadOnly Property Field() As Field
    Get
      Return _field
    End Get
  End Property
#End Region

#Region " Event Handlers "
  Private Sub btnOK_Click(ByVal sender As Object, ByVal e As EventArgs) _
      Handles btnOK.Click
    If _Validate() Then
      Me.DialogResult = Windows.Forms.DialogResult.OK
      Me.Close()
    End If
  End Sub

  Private Sub btnCancel_Click(ByVal sender As Object, ByVal e As EventArgs) _
      Handles btnCancel.Click
    Me.DialogResult = Windows.Forms.DialogResult.Cancel
    Me.Close()
  End Sub

  Private Sub txtFieldId_LostFocus(ByVal sender As Object, ByVal e As EventArgs) _
      Handles txtFieldId.LostFocus
    _Validate()
  End Sub
#End Region

#Region " Local Methods "
  Private Function _Validate() As Boolean
    Dim ret As Boolean

    erpErrors.SetError(txtFieldId, String.Empty)
    erpErrors.SetError(txtData, String.Empty)
    If Not _ValidateFieldId() Then
      erpErrors.SetError(txtFieldId, "Field ID must be a valid integer.")
    ElseIf txtData.Enabled AndAlso Not _ValidateFieldData() Then
      erpErrors.SetError(txtData, "Data must be in hexadecimal format with an even number of digits.")
    Else
      'I guess we're okay!
      ret = True
    End If

    Return ret
  End Function

  Private Function _ValidateFieldId() As Boolean
    Dim ret As Boolean = True

    Try
      _field.Id = Convert.ToInt32(txtFieldId.Text, 16)
      _field.Id = _field.Id And &HFFF0
    Catch
      ret = False
    End Try

    Return ret
  End Function

  Private Function _ValidateFieldData() As Boolean
    Dim ret As Boolean = True
    Dim data As String = txtData.Text.Replace(" "c, String.Empty)

    If data.Length Mod 2 <> 0 Then
      ret = False
    Else
      Try
        Dim d(Convert.ToInt32(data.Length / 2) - 1) As Byte

        For i As Integer = 0 To d.Length - 1
          d(i) = Convert.ToByte(data.Substring(0, 2), 16)

          data = data.Remove(0, 2)
        Next

        _field.Data = d
      Catch
        ret = False
      End Try
    End If

    Return ret
  End Function
#End Region

End Class
